# coding=utf8

import json


class MyDict(dict):
    """
    A **Python** _dict_ subclass which tries to act like **JavaScript** objects, so you can use the **dot notation** (.) to access members of the object. If the member doesn't exist yet then it's created when you assign something to it. Brackets notation (d['foo']) is also possible.
    """

    def __init__(self, dict_source=None, **kw):
        if dict_source and isinstance(dict_source, (dict, MyDict)):
            for k, v in dict_source.items():
                self[k] = self._transform(v)

        for key, value in kw.items():
            self[key] = self._transform(value)

    def _transform(self, source):
        if isinstance(source, (dict, MyDict)):
            return MyDict(source)

        elif isinstance(source, list):
            return [item for item in map(self._transform, source)]

        elif isinstance(source, tuple):
            result = None
            for item in source:

                if not result:
                    result = (self._transform(item),)

                else:
                    result = result + (self._transform(item),)

            return result

        else:
            # no need for transformation (int, float, str, set, ...)
            return source

    def __getattr__(self, name):
        """Get a field "name" from the object in the form: obj.name"""
        if name in self:
            return self[name]

    def __setattr__(self, name, value):
        """Sets a field into the object in the form obj.name = value"""
        self[name] = self._transform(value)

    def __getitem__(self, name):
        """Get a field "key" value from the object in the form: obj[name]"""
        return self.get(name)

    def get(self, key, default=None):
        if key in self:
            return super(MyDict, self).get(key, default)

        else:
            parts = str(key).split('.')
            if len(parts) > 1:
                try:
                    return self.get(parts[0]).get('.'.join(parts[1:]))

                except:
                    return None

            else:
                return super(MyDict, self).get(key, default)

    @staticmethod
    def from_json(json_string):
        """Returns a "MyDict" instance from a JSON string"""
        json_obj = json.loads(json_string)
        return MyDict(json_obj)



# coding=utf8

import decimal
from . import MyDict


class TestMyDict:

    def test__access_a_member_with_get(self):

        d = MyDict(foo='bar')
        assert d.get('foo') == 'bar'

    def test__access_a_member_with_get_and_a_path(self):
        """
        d = MyDict(foo={'bar': 'baz'})
        d.get('foo.bar') == 'baz'

        **Only dot notation 'foo.bar.baz'
        **Not supported yet things like: 'foo.bar[0].baz'
        """

        d = MyDict(foo={'bar': 'baz'})
        assert d.get('foo.bar') == 'baz'

    def test__access_a_member_with_get_and_a_path_with_brackets_notation(self):
        """
        d = MyDict(foo={'bar': 'baz'})
        d['foo.bar'] == 'baz'

        **Only dot notation 'foo.bar.baz'
        **Not supported yet things like: 'foo.bar[0].baz'
        """

        d = MyDict(foo={'bar': 'baz'})
        assert d['foo.bar'] == 'baz'

    def test__access_a_key_with_dots(self):
        """Access a key with dots in the name, where those dots don't represent a path"""

        d = MyDict({'foo.bar': 'baz', 'fuu': {'bar': 'baz'}})
        assert d['foo.bar'] == 'baz'
        assert d['fuu.bar'] == 'baz'

    def test__check_precedence_of_a_dotted_field_over_a_path(self):

        d = MyDict({'foo': {'bar': 'baz'}, 'foo.bar': 'BAZ'})

        assert d['foo.bar'] == 'BAZ'
        assert d.get('foo.bar') == 'BAZ'
        assert d.foo.bar == 'baz'

    def test__access_with_brackets_notation(self):
        """
        d = MyDict(foo='bar')
        d['foo'] == 'bar'
        """

        d = MyDict(foo='bar')
        assert d['foo'] == 'bar'

    def test__check_existence_with_in(self):

        d = MyDict(foo='bar')

        assert ('foo' in d) is True
        assert ('baz' in d) is False

    def test__items_method(self):

        d = MyDict(foo='Hello, MyDict!', bar=123, baz=[1, 2, 3])

        n = 0
        keys = {'foo', 'bar', 'baz'}
        for key, value in d.items():
            keys.remove(key)

            if key == 'foo':
                assert d.foo == 'Hello, MyDict!'

            elif key == 'bar':
                assert d.bar == 123

            elif key == 'baz':
                assert d.baz == [1, 2, 3]

            n += 1

        assert n == 3
        assert len(keys) == 0

    def test__new_member(self):

        d = MyDict()
        d.foo = 'This is a test'

        assert d.foo != None

    def test__non_existing_member(self):

        d = MyDict()
        assert d.foo is None

    def test__replacing_existing_member(self):

        d = MyDict()
        d.foo = 'bar'
        d.foo = 1

        assert d.foo != 'bar'
        assert d.foo == 1

    def test__replace_existing_member_with_object(self):

        d = MyDict()
        d.foo = 'bar'
        d.foo = {'bar': 'baz'}

        assert d.foo != 'bar'
        assert d.foo.bar == 'baz'

    def test__use_underscore_for_name(self):

        d = MyDict()
        d._foo = 'bar'
        d.__baz = 123

        assert d._foo == 'bar'
        assert d.__baz == 123

    def test__assign_str_value(self):

        d = MyDict()
        d.str_value = 'This is a test'

        assert d.str_value == 'This is a test'

    def test__assign_float_value(self):

        d = MyDict()
        d.float_value = 123.45

        assert d.float_value == 123.45

    def test__assign_int_value(self):

        d = MyDict()
        d.int_value = 123

        assert d.int_value == 123

    def test__assign_decimal_value(self):

        d = MyDict()
        d.decimal_value = decimal.Decimal(123)

        assert d.decimal_value == decimal.Decimal('123')

    def test__assign_list_value(self):

        d = MyDict()
        d.list_value = [1, 2, 3]

        assert d.list_value == [1, 2, 3]

    def test__access_a_list(self):

        d = MyDict()
        d.list_value = [1, 2, 3]

        assert d.list_value[1] == 2

    def test__assign_a_complex_list(self):

        d = MyDict()
        d.complex_list = [1, 2, {'foo': 'bar', 'baz': [1, 2, 3]}]

        assert d.complex_list[2].foo == 'bar'
        assert d.complex_list[2].baz[2] == 3

    def test__assign_a_tuple(self):

        d = MyDict()
        d.tuple_value = (1, 2, 3)

        assert d.tuple_value == (1, 2, 3)

    def test__access_a_tuple(self):

        d = MyDict()
        d.t = (1, 2, 3)

        assert d.t[1] == 2

    def test__assign_a_complex_tuple(self):

        d = MyDict()
        d.t = (1, 2, {'foo': 'bar', 'baz': [1, 2, 3]})

        assert d.t[2].foo == 'bar'
        assert d.t[2].baz[2] == 3

    def test__assign_a_dict_value(self):

        d = MyDict()
        d.dict_value = {'foo': 'bar', 'baz': 1}

        assert d.dict_value.foo == 'bar'
        assert d.dict_value.baz == 1

    def test__assign_a_mydict_value(self):

        d = MyDict()
        d.mydict = MyDict({'baz': 1}, foo='bar')

        assert d.mydict.baz == 1
        assert d.mydict.foo == 'bar'

    def test__assign_a_set_value(self):

        d = MyDict()
        d.assign_value = {1, 2, 3}

        assert d.assign_value == {1, 2, 3}

    def test__initialization_with_dict(self):

        d = MyDict({'foo': 'bar', 'baz': [1, 2, 3]})

        assert d.foo == 'bar'
        assert d.baz[2] == 3

    def test__initialization_with_mydict(self):

        d = MyDict(MyDict({'foo': 'bar', 'baz': 123}))
        assert d.foo == 'bar'
        assert d.baz == 123

    def test__initialization_with_keywords(self):

        d = MyDict(foo='bar', baz=1, a=2)

        assert d.foo == 'bar'
        assert d.baz == 1
        assert d.a == 2

    def test__initialization_with_dict_and_keywords(self):

        d = MyDict({'foo': 'bar'}, baz=[1, 2, 3])

        assert d.foo == 'bar'
        assert d.baz[1] == 2

    def test__initialization_with_keywords_overwrite(self):

        d = MyDict({'foo': 'bar'}, foo='BAR')
        assert d.foo == 'BAR'

    def test__complex_initialization(self):

        d = MyDict(foo={'bar': 'baz'}, lst=[1, decimal.Decimal('2.2'), {'foo': 'bar', 'baz': 1}, MyDict(foo='bar')])

        assert d.foo.bar == 'baz'
        assert d.lst[1] == decimal.Decimal('2.2')
        assert d.lst[2].foo == 'bar'
        assert d.lst[2].baz == 1
        assert d.lst[3].foo == 'bar'

    def test__dict_comparison(self):

        d = MyDict(foo='bar', baz=123)

        assert d == {'foo': 'bar', 'baz': 123}

    def test__create_from_json(self):

        d = MyDict.from_json('{"foo": "bar", "baz": [1, 2.2, "fuu", {"foo": "bar"}]}')

        assert d.foo == 'bar'
        assert d.baz[0] == 1
        assert d.baz[3].foo == 'bar'



# coding=utf8

from setuptools import setup

try:
    import pypandoc
    long_description = pypandoc.convert_file('./README.md', 'rst')

except ImportError:
    # when you're installing the package, to not force users to have "pypandoc" installed
    with open('./README.md', 'r') as f:
        long_description = f.read()

setup(
    name='mydict',
    version='1.0.6',
    author=u'León Domingo',
    author_email='leon@codevince.dev',
    description='A Python dict subclass which tries to act like JavaScript objects, so you can use the dot-notation (d.foo) to access members of the object.',
    long_description=long_description,
    license='MIT',
    keywords='dict javascript dot-notation codevince',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Utilities',
    ],
    url='https://github.com/codevincedev/mydict',
    packages=[
        'mydict',
    ],
    install_requires=[
        'pytest',
    ]
)
